/* $Id: wmpal.c 35 2007-07-05 14:05:18Z danadocus $ */
/* -*- indent-tabs-mode: nil; tab-width: 4; c-basic-offset: 4; -*- */
/*  wmpal - a useless DockApp
 *  Copyright (C) 2000      Hadess <hadess@writeme.com>
 *  Copyright (C) 2001-2004 A.Sleep <a.sleep@asleep.net>
 *  Copyright (C) 2007      Dana Jansens <danakj@orodu.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "xpms.h"

#include <X11/Xlib.h>
#include <X11/xpm.h>
#include <unistd.h> /* for sleep() */
#include <stdlib.h> /* for rand() */
#include <time.h>   /* for time() */
#include <string.h>
#include <stdio.h>

#define MAX_SLEEP_TIME 5

char *displayName = NULL;
char *palName = "dino";
int windowed = 0;
int nodecor = 0;
int below = 0;
int all = 0;

static void print_version();
static void print_help();
static void parse_args(int argc, char **argv);
static Window create_window(Display *d, const char *name, int w, int h,
                            int windowed, int nodecor, int below, int all);
static int load_pixmap(Display *d, char **data, Pixmap *xpm);

int main(int argc, char **argv)
{
    Display *d;
    Window win;
    Pixmap bg;
    Pixmap pixanim[3];

    parse_args(argc, argv);
    srand(time(NULL));

    if (!(d = XOpenDisplay(displayName))) {
        fprintf(stderr, "Unable to open the X display\n");
        exit(EXIT_FAILURE);
    }

    win = create_window(d, "wmpal", 50, 50, windowed, nodecor, below, all);

    switch (palName[0]) {
    case 'd':
        if (palName[1] == 'i') {
            load_pixmap(d, dino1_xpm, &pixanim[0]);
            load_pixmap(d, dino2_xpm, &pixanim[1]);
            load_pixmap(d, dino3_xpm, &pixanim[2]);
        }
        else {
            load_pixmap(d, demon1_xpm, &pixanim[0]);
            load_pixmap(d, demon2_xpm, &pixanim[1]);
            load_pixmap(d, demon3_xpm, &pixanim[2]);
        }
        break;
    case 'g':
        load_pixmap(d, grape1_xpm, &pixanim[0]);
        load_pixmap(d, grape2_xpm, &pixanim[1]);
        load_pixmap(d, grape3_xpm, &pixanim[2]);
        break;
    case 'p':
        load_pixmap(d, pear1_xpm, &pixanim[0]);
        load_pixmap(d, pear2_xpm, &pixanim[1]);
        load_pixmap(d, pear3_xpm, &pixanim[2]);
        break;
    case 's':
        if (palName[1] == 'u') {
            load_pixmap(d, sun1_xpm, &pixanim[0]);
            load_pixmap(d, sun2_xpm, &pixanim[1]);
            load_pixmap(d, sun3_xpm, &pixanim[2]);
        } else {
            load_pixmap(d, snail1_xpm, &pixanim[0]);
            load_pixmap(d, snail2_xpm, &pixanim[1]);
            load_pixmap(d, snail1_xpm, &pixanim[2]);
        }
        break;
    case 't':
        load_pixmap(d, turtle1_xpm, &pixanim[0]);
        load_pixmap(d, turtle2_xpm, &pixanim[1]);
        load_pixmap(d, turtle3_xpm, &pixanim[2]);
        break;
    }

    load_pixmap(d, bg_xpm, &bg);
    XSetWindowBackgroundPixmap(d, win, bg);
    XMapWindow(d, win);

    while (1) {
        static int anim_pos = -1;

        if (anim_pos < 2) ++anim_pos;
        else              anim_pos = 0;

        XCopyArea(d, pixanim[anim_pos], bg, DefaultGC(d, DefaultScreen(d)),
                  0, 0, 48, 48, 1, 1);
        XClearWindow(d, win);
        XFlush(d);

        /* sleep for a random time */
        sleep(rand() % (MAX_SLEEP_TIME-1) + 1);
    }

    return 0;
}

static void print_version()
{
    const char *version =
"wmpal " VERSION "\n"
"\n"
"Copyright (C) 2007      Dana Jansens <danakj@orodu.net>\n"
"Copyright (C) 2001-2004 A.Sleep <a.sleep@asleep.net>\n"
"Copyright (C) 2000      Hadess <hadess@writeme.com>\n"
"\n";

    printf("%s", version);
}

static void print_help()
{
    const char *help =
"Usage: wmpal [OPTIONS]\n"
"\n"
"  -d, --display <string>       Display to use\n"
"  -p, --pal <string>           Which pal to use\n"
"  -w, --windowed               runs the application in windowed mode\n"
"  -n, --nodecor                removes any decorations in windowed mode\n"
"  -b, --below                  keeps the window below others in windowed mode\n"
"  -a, --all                    show wmpal on all desktops\n"
"  -h, --help                   shows this help text and exit\n"
"  -v, --version                shows program version and exit\n"
"\n"
"Available pals are: demon, dino, grape, pear, snail, sun or turtle\n"
"\n";

    printf("%s", help);
}

static void parse_args(int argc, char **argv)
{
    int i;
    for (i = 1; i < argc; ++i) {
        if (!strcmp(argv[i], "-d") || !strcmp(argv[i], "--display")) {
            if (i == argc - 1) /* no args left */
                /* not translated cuz it's sekret */
                printf("--display requires an argument\n");
            else
                displayName = argv[++i];
        }
        else if (!strcmp(argv[i], "-p") || !strcmp(argv[i], "--pal")) {
            if (i == argc - 1) /* no args left */
                /* not translated cuz it's sekret */
                printf("--pal requires an argument\n");
            else
                palName = argv[++i];
        }
        else if (!strcmp(argv[i], "-w") || !strcmp(argv[i], "--windowed")) {
            windowed = 1;
        }
        else if (!strcmp(argv[i], "-n") || !strcmp(argv[i], "--nodecor")) {
            nodecor = 1;
        }
        else if (!strcmp(argv[i], "-b") || !strcmp(argv[i], "--below")) {
            below = 1;
        }
        else if (!strcmp(argv[i], "-a") || !strcmp(argv[i], "--all")) {
            all = 1;
        }
        else if (!strcmp(argv[i], "-h") || !strcmp(argv[i], "--help")) {
            print_help();
            exit(EXIT_SUCCESS);
        }
        else if (!strcmp(argv[i], "-v") || !strcmp(argv[i], "--version")) {
            print_version();
            exit(EXIT_SUCCESS);
        }
    }
}

#define MWM_FLAG_FUNCTIONS   (1 << 0)
#define MWM_FLAG_DECORATIONS (1 << 1)

static Window create_window(Display *d, const char *name, int w, int h,
                            int windowed, int nodecor, int below, int all)
{
    Window win;
    int s = DefaultScreen(d);

    win = XCreateWindow(d, RootWindow(d, s), 0, 0, w, h, 0,
                        DefaultDepth(d, s), InputOutput, DefaultVisual(d, s),
                        0, NULL);

    if (!windowed) {
        XWMHints *hint = XAllocWMHints();
        hint->flags = StateHint;
        hint->initial_state = WithdrawnState;
        XSetWMHints(d, win, hint);
        XFree(hint);
    }
    else {
        long state[3];
        int n = 0;

        if (nodecor) {
            long mwm[3];
            mwm[0] = MWM_FLAG_DECORATIONS;
            mwm[2] = 0; /* no decorations */
            XChangeProperty(d, win,
                            XInternAtom(d, "_MOTIF_WM_HINTS", False),
                            XInternAtom(d, "_MOTIF_WM_HINTS", False),
                            32, PropModeReplace, (unsigned char*) mwm, 3);

            /* set these when no decor is on */
            state[n++] = XInternAtom(d, "_NET_WM_STATE_SKIP_TASKBAR", False);
            state[n++] = XInternAtom(d, "_NET_WM_STATE_SKIP_PAGER", False);
        }

        if (below)
            state[n++] = XInternAtom(d, "_NET_WM_STATE_BELOW", False);
        if (n)
            XChangeProperty(d, win,
                            XInternAtom(d, "_NET_WM_STATE", False),
                            XInternAtom(d, "ATOM", False),
                            32, PropModeReplace, (unsigned char*) state, n);

        if (all) {
            long a[1] = { 0xffffffff };
            XChangeProperty(d, win,
                            XInternAtom(d, "_NET_WM_DESKTOP", False),
                            XInternAtom(d, "CARDINAL", False),
                            32, PropModeReplace, (unsigned char*) a, 1);
        }
    }

    {
        XClassHint c;
        c.res_name = "wmpal";
        c.res_class = "WMPal";
        XSetClassHint(d, win, &c);
    }

    return win;
}

static int load_pixmap(Display *d, char **data, Pixmap *xpm)
{
    Pixmap mask;
    unsigned short w, h;
    XpmAttributes at;
    
    at.valuemask = XpmCloseness;
    at.closeness = 40000;

    return XpmCreatePixmapFromData(d, RootWindow(d, DefaultScreen(d)),
                                   data, xpm, &mask, &at) == 0;
}

